--[[
	MyRolePlay 4 (C) 2010-2019 Etarna Moonshyne <etarna@moonshyne.org>, Katorie @ Moon Guard
	Licensed under GNU General Public Licence version 2 or, at your option, any later version

	Options.lua - Functions for handling base options, and version conversion
]]

local L = mrp.L
local strtrim = strtrim

local function emptynil( x ) return x ~= "" and x or nil end

mrp.DefaultOptions = {
	Enabled = true,
	ShowButton = true,
	ShowBiographyInBrowser = true,
	ShowTraitsInBrowser = true,
	HeightUnit = L["option_HeightUnit"],
	WeightUnit = L["option_WeightUnit"],
	FormAutoChange = true,
	AllowColours = true,
	TooltipClassColours = true,
	ClassNames = true,
	ShowOOC = true,
	ShowTarget = true,
	ShowVersion = true,
	HideTTInEncounters = false,
	ShowIconInTT = true,
	ShowGuildNames = true,
	ShowGlancePreview = true,
	MaxLinesSlider = 1,
	EquipSetAutoChange = true,
	TooltipStyle = 2,
	DEFontSize = 2,
	RPChatSay = true,
	RPChatWhisper = true,
	RPChatEmote = true,
	RPChatYell = true,
	RPChatParty = false,
	RPChatRaid = false,
	AutoplayMusic = false,
}



function mrp:UpgradeSaved( build )
	build = build or 0
	if build < 51 then
		-- Kill old temp unused options
		mrpSaved.Options.ImperialHeight = nil
		mrpSaved.Options.ImperialWeight = nil
		mrpSaved.Options.ShowInCombat = nil
		mrpSaved.Options.DisableInCombat = nil
		mrpSaved.Options.ShowInInstance = nil
		mrpSaved.Options.DisableInInstance = nil
		mrpSaved.Options.RelativeLevels = nil
		-- Set the new options to defaults
		mrpSaved.Options.Enabled = mrp.DefaultOptions.Enabled
		mrpSaved.Options.HeightUnit = mrp.DefaultOptions.HeightUnit
		mrpSaved.Options.WeightUnit = mrp.DefaultOptions.WeightUnit
	end
	if build < 52 then
		for name, profile in pairs( mrpSaved.Profiles ) do
			-- Strip spaces
			for field, contents in pairs( profile ) do
				profile[ field ] = strtrim( contents )
			end
 			-- Deal with people who put 01 in FR or FC
			if tonumber( profile.FR ) then
				profile.FR = tostring( tonumber( profile.FR ) )
			end
			if tonumber( profile.FC ) then
				profile.FC = tostring( tonumber( profile.FC ) )
			end
			-- RA should be blank if you aren't overriding, otherwise it messes up your race's localisation for other players
			if profile.RA == UnitRace("player") or profile.RA == select( 2, UnitRace("player") ) then
				profile.RA = ""
			end
			-- RC should be blank if you aren't overriding, otherwise it messes up your class' localisation for other players
			if profile.RC == UnitClass("player") or profile.RC ==  UnitClass("player") then
				profile.RC = ""
			end
		end
	end
	if build < 58 then
		mrpSaved.Options.FormAutoChange = mrp.DefaultOptions.FormAutoChange
	end
	if build < 59 then
		mrpSaved.Options.EquipSetAutoChange = mrp.DefaultOptions.EquipSetAutoChange
	end
	if build < 70 then
		mrpSaved.PreviousProfileAuto = nil
		mrpSaved.HumanForm = nil
	end
	if build < 73 then
		mrpSaved.Options.MRPButtonMoved = nil
		mrpSaved.Positions = {}
	end
	if build < 82 then
		if mrpSaved.Options.ShowTooltip then
			mrpSaved.Options.TooltipStyle = 2
		else
			mrpSaved.Options.TooltipStyle = 0
		end
		mrpSaved.Options.ShowTooltip = nil
	end
	if build < 86 then
		mrpSaved.Options.ShowRPNamesInChat = mrp.DefaultOptions.ShowRPNamesInChat
	end
	if build < 420 then
		mrpSaved.Options.TooltipStyle = 2;
		mrpSaved.Options.AllowColours = true;
		mrpSaved.Options.ClassNames = true;
		mrpSaved.Options.ShowOOC = true;
		mrpSaved.Options.ShowTarget = false;
		mrpSaved.Options.ShowVersion = true;
		mrpSaved.Options.ShowGuildNames = true;
		mrpSaved.Options.HideTTInEncounters = false;
		mrpSaved.Options.MaxLinesSlider = 1;
	end
	if build < 421 then
		mrpSaved.Options.ShowIconInTT = true;
	end
	if build < 422 then
		mrpSaved.Options.TooltipStyle = 2;
		C_Timer.After(10, function() mrp:Print("MRP now supports custom name, eye, description, and history colours. You can also set an icon for your profile and tooltip using the 'I' button in the profile editor."); end);
	end
	if build < 423 then
		mrpSaved.Options.ShowGlancePreview = true;
		C_Timer.After(10, function() mrp:Print("MRP now supports the 'At a Glance' field. You may set up to 5 glances, and include an icon. You can view other's glances (that have a supporting addon) via the viewer or the new (movable) preview box. You may disable the box entirely in options if you don't like it."); end);
	end
	if build < 424 then
		if(type(mrpNotes) ~= "table") then
			mrpNotes = {}
		end
		C_Timer.After(10, function() mrp:Print("You may now set a private note per profile that only you can see. Use the button by the close button in the profile viewer to set one."); end);
	end
	if build < 425 then
		C_Timer.After(10, function() mrp:Print("Version 425 includes a search box in the icon selector, along with various bug fixes."); end);
	end
	if build < 428 then
		mrpSaved.Options.AutoplayMusic = false;
		mrpSaved.Options.RPChatSay = true;
		mrpSaved.Options.RPChatWhisper = true;
		mrpSaved.Options.RPChatEmote = true;
		mrpSaved.Options.RPChatYell = true;
		mrpSaved.Options.RPChatParty = false;
		mrpSaved.Options.RPChatRaid = false;
		C_Timer.After(10, function() mrp:Print("You may now set music alongside your profile, click the 'Music' button on the profile editor. Check the patch notes on Curse for other changes."); end);
	end
	if build < 431 then
		local deSwapped = 0
		local hiSwapped = 0
		if(mrpSaved ~= nil and mrpSaved.Profiles ~= nil) then
			for k, v in pairs(mrpSaved.Profiles) do
				if(mrpSaved.Profiles[k]["descripColour"] ~= nil) then
					rgbColour = CreateColor(mrpSaved.Profiles[k]["descripColour"]["r"], mrpSaved.Profiles[k]["descripColour"]["g"], mrpSaved.Profiles[k]["descripColour"]["b"]);
					hexcode = rgbColour:GenerateHexColorMarkup()
					mrpSaved.Profiles[k]["DE"] = (hexcode .. mrpSaved.Profiles[k]["DE"] .. "|r")
					mrpSaved.Profiles[k]["descripColour"] = nil
					deSwapped = (deSwapped + 1)
				end;
			end;
			for k, v in pairs(mrpSaved.Profiles) do
				if(mrpSaved.Profiles[k]["historyColour"] ~= nil) then
					rgbColour = CreateColor(mrpSaved.Profiles[k]["historyColour"]["r"], mrpSaved.Profiles[k]["historyColour"]["g"], mrpSaved.Profiles[k]["historyColour"]["b"]);
					hexcode = rgbColour:GenerateHexColorMarkup()
					mrpSaved.Profiles[k]["HI"] = (hexcode .. mrpSaved.Profiles[k]["HI"] .. "|r")
					mrpSaved.Profiles[k]["historyColour"] = nil
					hiSwapped = (hiSwapped + 1)
				end;
			end;
		end;
		C_Timer.After(10, function() mrp:Print("The colour system has been upgraded for description and history fields to support multiple colours. Check the patch notes on Curse for details."); end);
		C_Timer.After(11, function() mrp:Print(deSwapped .. " descriptions automatically transferred to new system."); end);
		C_Timer.After(11.2, function() mrp:Print(hiSwapped .. " histories automatically transferred to new system."); end);
	end
	if build < 432 then
		mrpSaved.Options.ShowTraitsInBrowser = true;
		mrpSaved.Options.TooltipClassColours = true;
		mrpSaved.Options.DEFontSize = 2;
		C_Timer.After(10, function() mrp:Print("MRP now supports personality traits. Check out the new tab in the profile viewer and set your own in the editor. You may disable this new tab in options if you'd like."); end);
	end
end